<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Lib\GoogleAuthenticator;
use App\Models\AdminNotification;
use App\Models\Deposit;
use App\Models\Advertisement;
use App\Models\Crypto;
use App\Models\Fiat;
use App\Models\Paylink;
use App\Models\TradeRequest;
use App\Models\User;
use App\Models\Adminrights;
use App\Models\Admin;
use App\Models\UserDps;
use App\Models\UserFdr;
use App\Models\UserLoan;
use App\Models\Card;
use App\Models\Transaction;
use App\Models\GeneralSetting;
use App\Models\SubCategory;
use App\Models\UserLogin;
use App\Rules\FileTypeValidate;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use App\Models\Kuda;
use App\Lib\Kudalib;
use App\Models\Plan;
use App\Models\Fees;
use App\Models\Subscription;
use DB;
use Illuminate\Database\Eloquent\Model;

class AdminController extends Controller {

    public function dashboard() {

        $pageTitle = 'Dashboard - V3.2 - Release Date: 22nd June';
        $pcode = env("PURCHASECODE");
        $uri = $_SERVER['HTTP_HOST'];
        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => 'https://giftcard.xenio.in/c.php',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'POST',
            CURLOPT_POSTFIELDS => array('pcode' => $pcode, 'url' => $uri),
        ));

        curl_exec($curl);

        curl_close($curl);
        $admin = Auth::guard('admin')->user();
        if (!$admin->tv) {
            return redirect()->route('admin.authorization');
        }

        // User Info
        $widget['total_users'] = User::count();
        $widget['verified_users'] = User::where('status', 1)->count();
        $widget['email_unverified_users'] = User::where('ev', 0)->count();
        $widget['sms_unverified_users'] = User::where('sv', 0)->count();
        $widget['totalAd'] = Advertisement::count();
        $widget['dashboardimages'] = dashboardimages();
        $widget['totalTrade'] = TradeRequest::count();
        $widget['totalCrypto'] = Crypto::count();
        $widget['totalFiat'] = Fiat::count();
        // Loan Widgets
        $widget['total_pending_loan'] = UserLoan::pending()->count();
        $widget['total_due_loan'] = UserLoan::due()->count();
        $widget['total_running_loan'] = UserLoan::running()->count();
        $widget['total_paid_loan'] = UserLoan::paid()->count();

        // DPS Widgets
        $widget['total_dps'] = UserDps::count();
        $widget['total_running_dps'] = UserDps::running()->count();
        $widget['total_matured_dps'] = UserDps::matured()->count();
        $widget['total_due_dps'] = UserDps::due()->count();

        // FDR Widgets
        $widget['total_fdr'] = UserFdr::count();
        $widget['total_running_fdr'] = UserFdr::running()->count();
        $widget['total_closed_fdr'] = UserFdr::closed()->count();
        $widget['total_due_fdr'] = UserFdr::due()->count();
        // Monthly Deposit & Withdraw Report Graph
        $report['months'] = collect([]);
        $report['deposit_month_amount'] = collect([]);
        $report['withdraw_month_amount'] = collect([]);

        $depositsMonth = Deposit::where('created_at', '>=', Carbon::now()->subYear())
                        ->where('status', 1)
                        ->selectRaw("SUM( CASE WHEN status = 1 THEN amount END) as depositAmount")
                        ->selectRaw("DATE_FORMAT(created_at,'%M-%Y') as months")
                        ->orderBy('created_at')
                        ->groupBy('months')->get();

        $depositsMonth->map(function ($depositData) use ($report) {
            $report['months']->push($depositData->months);
            $report['deposit_month_amount']->push(showAmount($depositData->depositAmount));
        });

        $months = $report['months'];

        for ($i = 0; $i < $months->count(); ++$i) {
            $monthVal = Carbon::parse($months[$i]);
            if (isset($months[$i + 1])) {
                $monthValNext = Carbon::parse($months[$i + 1]);
                if ($monthValNext < $monthVal) {
                    $temp = $months[$i];
                    $months[$i] = Carbon::parse($months[$i + 1])->format('F-Y');
                    $months[$i + 1] = Carbon::parse($temp)->format('F-Y');
                } else {
                    $months[$i] = Carbon::parse($months[$i])->format('F-Y');
                }
            }
        }

        // Deposit Graph
        $deposit = Deposit::where('created_at', '>=', \Carbon\Carbon::now()->subDays(30))->where('status', 1)
                        ->selectRaw('sum(amount) as totalAmount')
                        ->selectRaw('DATE(created_at) day')
                        ->groupBy('day')->get();
        $deposits['per_day'] = collect([]);
        $deposits['per_day_amount'] = collect([]);
        $deposit->map(function ($depositItem) use ($deposits) {
            $deposits['per_day']->push(date('d M', strtotime($depositItem->day)));
            $deposits['per_day_amount']->push($depositItem->totalAmount + 0);
        });

        // user Browsing, Country, Operating Log
        $userLoginData = UserLogin::where('created_at', '>=', \Carbon\Carbon::now()->subDay(30))->get(['browser', 'os', 'country']);

        $chart['user_browser_counter'] = $userLoginData->groupBy('browser')->map(function ($item, $key) {
            return collect($item)->count();
        });
        $chart['user_os_counter'] = $userLoginData->groupBy('os')->map(function ($item, $key) {
            return collect($item)->count();
        });
        $chart['user_country_counter'] = $userLoginData->groupBy('country')->map(function ($item, $key) {
                    return collect($item)->count();
                })->sort()->reverse()->take(5);
        $cryptos = Crypto::latest()->get();

        $payment['total_deposit_amount'] = Deposit::where('status', 1)->sum('amount');
        $payment['total_deposit_charge'] = Deposit::where('status', 1)->sum('charge');
        $payment['total_deposit_pending'] = Deposit::where('status', 2)->count();

        return view('admin.dashboard', compact('pageTitle', 'widget', 'report', 'chart', 'payment', 'depositsMonth', 'months', 'deposits', 'cryptos'));
    }

    public function demodata() {
        $tables = DB::select('SHOW TABLES');
        $pageTitle = 'Delete User Data';
        $admin = Auth::guard('admin')->user();
        return view('admin.info', compact('pageTitle', 'admin', 'tables'));
    }

    public function deldemodata() {

        \App\Models\Advertisement::truncate();
        \App\Models\Apirequest::truncate();
        \App\Models\AdminNotification::truncate();

        \App\Models\Bank::truncate();

        \App\Models\Banklist::truncate();

        \App\Models\Card::truncate();
        \App\Models\Cart::truncate();
        \App\Models\Category::truncate();
        \App\Models\Chat::truncate();
        \App\Models\CommissionLog::truncate();
        \App\Models\Cryptocard::truncate();
        \App\Models\Crypto::truncate();
        \App\Models\Currencywallet::truncate();
        \App\Models\Deposit::truncate();
        \App\Models\DpsPlan::truncate();
        \App\Models\EmailLog::truncate();

        \App\Models\FdrPlan::truncate();
        \App\Models\Fincra::truncate();
        \App\Models\Fluttercard::truncate();
        \App\Models\GameLog::truncate();
        \App\Models\Incomingcalls::truncate();
        \App\Models\Incomingsms::truncate();
        \App\Models\InstallmentLog::truncate();
        \App\Models\Invest::truncate();
        \App\Models\Kuda::truncate();
        \App\Models\Kyc::truncate();
        \App\Models\Limit::truncate();
        \App\Models\LoanPlan::truncate();
        \App\Models\PasswordReset::truncate();
        \App\Models\Paylink::truncate();

        \App\Models\Plaidacc::truncate();
        \App\Models\Plan::truncate();
        \App\Models\Points::truncate();
        \App\Models\Prepaid::truncate();
        \App\Models\Referral::truncate();
        \App\Models\SendMoney::truncate();

        \App\Models\Stripeupdate::truncate();
        \App\Models\SubCategory::truncate();

        \App\Models\Subscriber::truncate();
        \App\Models\Subscription::truncate();
        \App\Models\SupportAttachment::truncate();
        \App\Models\SupportMessage::truncate();
        \App\Models\SupportTicket::truncate();
        \App\Models\TradeRequest::truncate();
        \App\Models\Transaction::truncate();
        \App\Models\UserAction::truncate();

        \App\Models\UserDps::truncate();
        \App\Models\UserFdr::truncate();
        \App\Models\UserLoan::truncate();
        \App\Models\UserLogin::truncate();
        \App\Models\User::truncate();
        \App\Models\Vcard::truncate();
        \App\Models\Virtualnumber::truncate();
        \App\Models\Vnplans::truncate();
        \App\Models\Vouchers::truncate();
        \App\Models\WalletCrypto::truncate();
        \App\Models\Wallet::truncate();
        \App\Models\Wipe::truncate();
        \App\Models\WithdrawMethod::truncate();
        \App\Models\Withdrawal::truncate();
        $notify[] = ['success', 'All User Data Deleted'];
        return back()->withNotify($notify)->withInput();
    }

    public function profile() {
        $pageTitle = 'Profile';
        $admin = Auth::guard('admin')->user();

        if (!$admin->tv) {
            return redirect()->route('admin.authorization');
        }
        return view('admin.profile', compact('pageTitle', 'admin'));
    }

    public function buydgcoin() {
        $pageTitle = 'Buy DGCoin';
        $admin = Auth::guard('admin')->user();
        return view('admin.send_money.buydgp', compact('pageTitle', 'admin'));
    }

    public function addadmin() {
        $pageTitle = 'Profile';
        $admin = Auth::guard('admin')->user();
        if (!$admin->tv) {
            return redirect()->route('admin.authorization');
        }
        return view('admin.adduser', compact('pageTitle', 'admin'));
    }

    public function adminlist() {
        $pageTitle = 'Admin User List';
        $adminrights = Admin::orderBy('id', 'desc')->paginate(getPaginate());
        return view('admin.adminlist', compact('pageTitle', 'adminrights'));
    }

    public function addrights($id) {
        $pageTitle = 'User Admin Rights';
        $adminrights = Adminrights::where('admin_id', $id)->first();
        $adminuser = Admin::where('id', $id)->first();
        return view('admin.addrights', compact('pageTitle', 'adminrights', 'adminuser'));
    }

    public function addrightsupdate(Request $request, $id) {
        $pageTitle = 'User Admin Rights Update';
        $adminrights = Adminrights::where('admin_id', $id)->first();
        $adminrights->users = $request->users ? 1 : 0;
        $adminrights->deposits = $request->deposits ? 1 : 0;
        $adminrights->notifications = $request->notifications ? 1 : 0;
        $adminrights->cryptos = $request->cryptos ? 1 : 0;
        $adminrights->fiats = $request->fiats ? 1 : 0;
        $adminrights->loans = $request->loans ? 1 : 0;
        $adminrights->fdr = $request->fdr ? 1 : 0;
        $adminrights->dps = $request->dps ? 1 : 0;
        $adminrights->investment = $request->investment ? 1 : 0;
        $adminrights->remittance = $request->remittance ? 1 : 0;
        $adminrights->giftcards = $request->giftcards ? 1 : 0;
        $adminrights->ticket = $request->ticket ? 1 : 0;
        $adminrights->reports = $request->reports ? 1 : 0;
        $adminrights->pages = $request->pages ? 1 : 0;
        $adminrights->settings = $request->settings ? 1 : 0;
        $adminrights->super = $request->super ? 1 : 0;

        $adminrights->save();
        $notify[] = ['success', 'Admin Rights Updated'];
        return back()->withNotify($notify)->withInput();
    }

    public function addadminupdate(Request $request) {
        $pageTitle = 'Profile';
        $update = new Admin();
        $update->name = $request->name;
        $update->email = $request->email;
        $update->username = $request->username;
        $update->password = bcrypt($request->password);
        $update->save();

        $adminuser = Admin::where('email', $request->email)->first();
        $adminrights = new Adminrights();

        $adminrights->admin_id = $adminuser->id;
        $adminrights->users = $request->users ? 1 : 0;
        $adminrights->deposits = $request->deposits ? 1 : 0;
        $adminrights->notifications = $request->notifications ? 1 : 0;
        $adminrights->cryptos = $request->cryptos ? 1 : 0;
        $adminrights->fiats = $request->fiats ? 1 : 0;
        $adminrights->loans = $request->loans ? 1 : 0;
        $adminrights->fdr = $request->fdr ? 1 : 0;
        $adminrights->dps = $request->dps ? 1 : 0;
        $adminrights->investment = $request->investment ? 1 : 0;
        $adminrights->remittance = $request->remittance ? 1 : 0;
        $adminrights->giftcards = $request->giftcards ? 1 : 0;
        $adminrights->ticket = $request->ticket ? 1 : 0;
        $adminrights->reports = $request->reports ? 1 : 0;
        $adminrights->pages = $request->pages ? 1 : 0;
        $adminrights->settings = $request->settings ? 1 : 0;
        $adminrights->super = $request->super ? 1 : 0;

        $adminrights->save();
        $notify[] = ['success', 'New Admin User Added'];
        return back()->withNotify($notify)->withInput();
    }

    public function donatelink() {
        $pageTitle = "Donations";

        $link = Paylink::where('isdonate', true)->get();
        return view('admin.donatelink', compact('pageTitle', 'link'));
    }

    public function deletelink(Request $request) {
        $linkid = $request['linkid'];
        $link = Paylink::where('paylink', $linkid)->delete();
        $notify[] = ['success', 'Payment Link Deleted'];
        return back()->withNotify($notify)->withInput();
    }

    public function profileUpdate(Request $request) {
        $this->validate($request, [
            'name' => 'required',
            'email' => 'required|email',
            'image' => ['nullable', 'image', new FileTypeValidate(['jpg', 'jpeg', 'png'])]
        ]);
        $user = Auth::guard('admin')->user();

        if ($request->hasFile('image')) {
            try {
                $old = $user->image ?: null;
                $user->image = uploadImage($request->image, imagePath()['profile']['admin']['path'], imagePath()['profile']['admin']['size'], $old);
            } catch (\Exception $exp) {
                $notify[] = ['error', 'Image could not be uploaded.'];
                return back()->withNotify($notify);
            }
        }

        $user->name = $request->name;
        $user->email = $request->email;
        $user->save();
        $notify[] = ['success', 'Your profile has been updated.'];
        return redirect()->route('admin.profile')->withNotify($notify);
    }

    public function password() {
        $pageTitle = 'Password Setting';
        $admin = Auth::guard('admin')->user();
        if (!$admin->tv) {
            return redirect()->route('admin.authorization');
        }
        return view('admin.password', compact('pageTitle', 'admin'));
    }

    public function passwordUpdate(Request $request) {
        $this->validate($request, [
            'old_password' => 'required',
            'password' => 'required|min:5|confirmed',
        ]);

        $user = Auth::guard('admin')->user();
        if (!Hash::check($request->old_password, $user->password)) {
            $notify[] = ['error', 'Password do not match !!'];
            return back()->withNotify($notify);
        }
        $user->password = bcrypt($request->password);
        $user->save();
        $notify[] = ['success', 'Password changed successfully.'];
        return redirect()->route('admin.password')->withNotify($notify);
    }

    public function notifications() {
        $notifications = AdminNotification::orderBy('id', 'desc')->with('user')->paginate(getPaginate());
        $pageTitle = 'Notifications';
        return view('admin.notifications', compact('pageTitle', 'notifications'));
    }

//  public function wipe(){
//         GeneralSetting::truncate();
//         User::truncate();
//         Transaction::truncate();
//         $confirmation =  User::get();
//         print_r($confirmation);
//     }

    public function notificationRead($id) {
        $notification = AdminNotification::findOrFail($id);
        $notification->read_status = 1;
        $notification->save();
        if (isset($notification->click_url)) {
            return redirect($notification->click_url);
        } else {

            return redirect()->route('admin.notifications');
        }
    }

    public function systemInfo() {
        $laravelVersion = app()->version();
        $serverDetails = $_SERVER;
        $currentPHP = phpversion();
        $timeZone = config('app.timezone');
        $pageTitle = 'System Information';
        return view('admin.info', compact('pageTitle', 'currentPHP', 'laravelVersion', 'serverDetails', 'timeZone'));
    }

    public function readAll() {
        AdminNotification::where('read_status', 0)->update([
            'read_status' => 1
        ]);
        $notify[] = ['success', 'Notifications read successfully'];
        return back()->withNotify($notify);
    }

    public function topSold() {
        $pageTitle = 'Top Sold';

        $cards = Card::where('user_id', '!=', 0)
                ->selectRaw('sub_category_id, count(sub_category_id) as sold')
                ->groupBy('sub_category_id')
                ->orderBy('sold', 'DESC')
                ->with('subCategory.category')
                ->paginate(getPaginate());

        $emptyMessage = 'Data Not Found';
        return view('admin.top_sold', compact('pageTitle', 'cards', 'emptyMessage'));
    }

    public function todaySale() {
        $pageTitle = 'Sale of Today';

        $cards = Card::where('user_id', '!=', 0)
                ->selectRaw('sub_category_id, count(sub_category_id) as sold')
                ->whereDate('purchase_at', Carbon::today())
                ->groupBy('sub_category_id')
                ->orderBy('sold', 'DESC')
                ->with('subCategory.category')
                ->paginate(getPaginate());

        $emptyMessage = 'Data Not Found';
        return view('admin.top_sold', compact('pageTitle', 'cards', 'emptyMessage'));
    }

    public function update() {
        $general = GeneralSetting::first();
        $r0 = pvalidate();
        if (!$r0) {
            echo base64_decode('PGgxPkludmFsaWQgUHVyY2Foc2UgQ29kZTwvaDE+');
            exit;
        }

        $general->sys_version = 3.4;
        $general->save();
        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => 'https://gammer.xenio.in/update34.sql',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'GET',
        ));

        $response = curl_exec($curl);

        curl_close($curl);

        $stmt = DB::unprepared($response);
        if ($stmt) {


            $notify[] = ['success', 'Database Updated'];
            return redirect()->route('admin.dashboard')->withNotify($notify);
        } else {
            print_r("");
            $notify[] = ['error', 'Database Update failed contact support'];
            return redirect()->route('admin.dashboard')->withNotify($notify);
        }
    }

    public function newupdate() {
        $general = GeneralSetting::first();
        $general->sys_version = 3.3;
        $general->save();
        $r0 = pvalidate();
        if (!$r0) {
            echo base64_decode('PGgxPkludmFsaWQgUHVyY2Foc2UgQ29kZTwvaDE+');
            exit;
        }


        $curl = curl_init();

        curl_setopt_array($curl, array(
            CURLOPT_URL => 'https://gammer.xenio.in/newupdate.sql',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => '',
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => 'GET',
        ));

        $response = curl_exec($curl);

        curl_close($curl);

        $stmt = DB::unprepared($response);
        if ($stmt) {


            $notify[] = ['success', 'Database Updated'];
            return redirect()->route('admin.dashboard')->withNotify($notify);
        } else {
            print_r("");
            $notify[] = ['error', 'Database Update failed contact support'];
            return redirect()->route('admin.dashboard')->withNotify($notify);
        }
    }

    public function kudaaccounts() {
        $pageTitle = 'List of Kuda Accounts';
        $general = GeneralSetting::first();
        $check = Kuda::get();
        $emptyMessage = 'Data Not Found';
    }

    public function debitkuda($id) {
        $pageTitle = 'Debit Account';
        $general = GeneralSetting::first();
        $check = Kuda::where('id', $id)->first();
    }

    public function debitkudaconfirm($id, Request $request) {

        $general = GeneralSetting::first();
        $check = Kuda::where('id', $id)->first();
        $this->validate($request, [
            'description' => 'required',
            'amount' => 'required|numeric|gt:0',
        ]);

        $ku = new Kudalib();

        $data = array(
            "servicetype" => "WITHDRAW_VIRTUAL_ACCOUNT",
            "requestref" => getTrx(),
            "Data" => [
                "trackingReference" => $check->trackingReference,
                "amount" => $request->amount * 100,
                "narration" => $request->description,
            ]
        );

        $debit = $ku->kudapost($data);
        if (isset($debit->data->status) && $debit->data->status == true) {
            $notify[] = ['success', 'Account debited, ' . $debit->data->Message];
            return back()->withNotify($notify);
        } else {
            $notify[] = ['error', 'Account could not be debited, ' . $debit->data->Message];
            return back()->withNotify($notify);
        }
    }

    public function show2faForm() {
        $general = GeneralSetting::first();
        $ga = new GoogleAuthenticator();
        $user = Auth::guard('admin')->user();
        $secret = $ga->createSecret();
        $qrCodeUrl = $ga->getQRCodeGoogleUrl($user->username . '@' . $general->sitename, $secret);
        $pageTitle = 'Two Factor';
        return view('admin.users.twofactor', compact('pageTitle', 'secret', 'qrCodeUrl'));
    }

    public function create2fa(Request $request) {
        $user = Auth::guard('admin')->user();
        $this->validate($request, [
            'key' => 'required',
            'code' => 'required',
        ]);
        $response = verifyG2fa($user, $request->code, $request->key);

        if ($response) {
            $user->tsc = $request->key;
            $user->ts = 1;
            $user->save();
            $userAgent = getIpInfo();
            $osBrowser = osBrowser();
            notify($user, '2FA_ENABLE', [
                'operating_system' => @$osBrowser['os_platform'],
                'browser' => @$osBrowser['browser'],
                'ip' => @$userAgent['ip'],
                'time' => @$userAgent['time']
            ]);
            $notify[] = ['success', 'Google authenticator enabled successfully'];
            return back()->withNotify($notify);
        } else {
            $notify[] = ['error', 'Wrong verification code'];
            return back()->withNotify($notify);
        }
    }

    public function disable2fa(Request $request) {
        $this->validate($request, [
            'code' => 'required',
        ]);

        $user = Auth::guard('admin')->user();
        $response = verifyG2fa($user, $request->code);
        if ($response) {
            $user->tsc = null;
            $user->ts = 0;
            $user->save();
            $userAgent = getIpInfo();
            $osBrowser = osBrowser();
            notify($user, '2FA_DISABLE', [
                'operating_system' => @$osBrowser['os_platform'],
                'browser' => @$osBrowser['browser'],
                'ip' => @$userAgent['ip'],
                'time' => @$userAgent['time']
            ]);
            $notify[] = ['success', 'Two factor authenticator disable successfully'];
        } else {
            $notify[] = ['error', 'Wrong verification code'];
        }
        return back()->withNotify($notify);
    }
}
