<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Frontend;
use App\Models\GeneralSetting;
use App\Rules\FileTypeValidate;
use Illuminate\Http\Request;
use App\Models\Holiday;
use Illuminate\Support\Facades\Artisan;
use Image;

class GeneralSettingController extends Controller {

    public function index() {
        $r0 = pvalidate();
        if (!$r0) {
            echo base64_decode('PGgxPkludmFsaWQgUHVyY2Foc2UgQ29kZTwvaDE+');
            exit;
        }
        $general = GeneralSetting::first();
        $pageTitle = 'General Setting';
        $timezones = json_decode(file_get_contents(resource_path('views/admin/partials/timezone.json')));
        return view('admin.setting.general_setting', compact('pageTitle', 'general', 'timezones'));
    }

    public function update(Request $request) {
        $r0 = pvalidate();
        if (!$r0) {
            echo base64_decode('PGgxPkludmFsaWQgUHVyY2Foc2UgQ29kZTwvaDE+');
            exit;
        }
        $request->validate([
            'base_color' => 'nullable', 'regex:/^[a-f0-9]{6}$/i',
            'secondary_color' => 'nullable', 'regex:/^[a-f0-9]{6}$/i',
            'timezone' => 'required',
        ]);

        $general = GeneralSetting::first();

        $general->dark = $request->dark ? 1 : 0;

        $general->sitename = $request->sitename;
        $general->cur_text = $request->cur_text;
        $general->cur_sym = $request->cur_sym;
        $general->base_color = $request->base_color;
        $general->secondary_color = $request->secondary_color;

        $general->usd_fee = $request->usd_fee;
        $general->ngn_fee = $request->ngn_fee;
        $general->pg_fee = $request->pg_fee;
        $general->tx_fee = $request->tx_fee;
        $general->ev_fee = $request->ev_fee;
        $general->doc = $request->doc;

        $general->prepaidissue_fee = isset($request->prepaidissue_fee) ? $request->prepaidissue_fee : 0;

        $general->load_fee = isset($request->load_fee) ? $request->load_fee : 0;
        $general->cryptoload_fee = isset($request->cryptoload_fee) ? $request->cryptoload_fee : 0;
        $general->crypto_masterfee = isset($request->crypto_masterfee) ? $request->crypto_masterfee : 0;
        $general->ccard_fee = isset($request->ccard_fee) ? $request->ccard_fee : 0;

        $general->visa_fee = isset($request->visa_fee) ? $request->visa_fee : 0;
        $general->visa_loadfee = isset($request->visa_loadfee) ? $request->visa_loadfee : 0;
        $general->usdvisa_fee = isset($request->usdvisa_fee) ? $request->usdvisa_fee : 0;
        $general->usdvisa_loadfee = isset($request->usdvisa_loadfee) ? $request->usdvisa_loadfee : 0;

        $general->card_bin = isset($request->card_bin) ? $request->card_bin : "7249";
        $general->user_cashout = isset($request->user_cashout) ? $request->user_cashout : 0;
        $general->user_cashin = isset($request->user_cashin) ? $request->user_cashin : 0;
        $general->agent_cashpay = isset($request->agent_cashpay) ? $request->agent_cashpay : 0;
        $general->agent_cashrcv = isset($request->agent_cashrcv) ? $request->agent_cashrcv : 0;

        $general->physical_fee = isset($request->physical_fee) ? $request->physical_fee : 0;

        $general->crypto_monthly = isset($request->crypto_monthly) ? $request->crypto_monthly : 0;
        $general->fiat_monthly = isset($request->fiat_monthly) ? $request->fiat_monthly : 0;

        $general->bsiissue_fee = isset($request->bsiissue_fee) ? $request->bsiissue_fee : 0;

        $general->bsiload_fee = isset($request->bsiload_fee) ? $request->bsiload_fee : 0;

        $general->save();

        $timezoneFile = config_path('timezone.php');
        $content = '<?php $timezone = ' . $request->timezone . ' ?>';
        file_put_contents($timezoneFile, $content);
        $notify[] = ['success', 'General setting has been updated.'];
        return back()->withNotify($notify);
    }

    public function logoIcon() {
        $r0 = pvalidate();
        if (!$r0) {
            echo base64_decode('PGgxPkludmFsaWQgUHVyY2Foc2UgQ29kZTwvaDE+');
            exit;
        }
        $pageTitle = 'Logo & Favicon';
        return view('admin.setting.logo_icon', compact('pageTitle'));
    }

    public function logoIconUpdate(Request $request) {
        $request->validate([
            'logo' => ['image', new FileTypeValidate(['jpg', 'jpeg', 'png'])],
            'darkLogo' => ['image', new FileTypeValidate(['jpg', 'jpeg', 'png'])],
            'favicon' => ['image', new FileTypeValidate(['png'])],
        ]);

        if ($request->hasFile('logo')) {
            try {
                $path = imagePath()['logoIcon']['path'];
                if (!file_exists($path)) {
                    mkdir($path, 0755, true);
                }

                Image::make($request->logo)->save($path . '/logo.png');
            } catch (\Exception $exp) {
                $notify[] = ['error', 'Logo could not be uploaded.'];
                return back()->withNotify($notify);
            }
        }

        if ($request->hasFile('darkLogo')) {
            try {
                $path = imagePath()['logoIcon']['path'];
                if (!file_exists($path)) {
                    mkdir($path, 0755, true);
                }
                Image::make($request->darkLogo)->save($path . '/darkLogo.png');
            } catch (\Exception $exp) {
                $notify[] = ['error', 'Dark Logo could not be uploaded.'];
                return back()->withNotify($notify);
            }
        }

        if ($request->hasFile('favicon')) {
            try {
                $path = imagePath()['logoIcon']['path'];
                if (!file_exists($path)) {
                    mkdir($path, 0755, true);
                }
                $size = explode('x', imagePath()['favicon']['size']);
                Image::make($request->favicon)->resize($size[0], $size[1])->save($path . '/favicon.png');
            } catch (\Exception $exp) {
                $notify[] = ['error', 'Favicon could not be uploaded.'];
                return back()->withNotify($notify);
            }
        }
        $notify[] = ['success', 'Logo & favicon has been updated.'];
        return back()->withNotify($notify);
    }

    public function customCss() {
        $pageTitle = 'Custom CSS';
        $file = activeTemplate(true) . 'css/custom.css';
        $file_content = @file_get_contents($file);
        return view('admin.setting.custom_css', compact('pageTitle', 'file_content'));
    }

    public function customCssSubmit(Request $request) {
        $file = activeTemplate(true) . 'css/custom.css';
        if (!file_exists($file)) {
            fopen($file, "w");
        }
        file_put_contents($file, $request->css);
        $notify[] = ['success', 'CSS updated successfully.'];
        return back()->withNotify($notify);
    }

    public function optimize() {
        Artisan::call('optimize:clear');
        $notify[] = ['success', 'Cache cleared successfully'];
        return back()->withNotify($notify);
    }

    public function cookie() {
        $pageTitle = 'GDPR Cookie';
        $cookie = Frontend::where('data_keys', 'cookie.data')->firstOrFail();
        return view('admin.setting.cookie', compact('pageTitle', 'cookie'));
    }

    public function cookieSubmit(Request $request) {
        $request->validate([
            'link' => 'required',
            'description' => 'required',
        ]);
        $cookie = Frontend::where('data_keys', 'cookie.data')->firstOrFail();
        $cookie->data_values = [
            'link' => $request->link,
            'description' => $request->description,
            'status' => $request->status ? 1 : 0,
        ];
        $cookie->save();
        $notify[] = ['success', 'Cookie policy updated successfully.'];
        return back()->withNotify($notify);
    }

    public function apiIndex() {
        $pageTitle = 'Api Setting';
        $general = GeneralSetting::first();

        return view('admin.setting.api_setting', compact('pageTitle', 'general'));
    }

    public function moduleIndex() {
        $pageTitle = 'Module Setting';
        $general = GeneralSetting::first();

        return view('admin.setting.module_setting', compact('pageTitle', 'general'));
    }

    public function mobilemoduleIndex() {
        $pageTitle = 'Module Setting';
        $general = GeneralSetting::first();

        return view('admin.setting.mobilemodule_setting', compact('pageTitle', 'general'));
    }

    public function mobilemoduleUpdate(Request $request) {


        $general = GeneralSetting::first();

        $general->loan = $request->loan ? 1 : 0;
        $general->dps = $request->dps ? 1 : 0;
        $general->fdr = $request->fdr ? 1 : 0;
        $general->investments = $request->investments ? 1 : 0;
        $general->payment_deposit = $request->payment_deposit ? 1 : 0;
        $general->user_transfer = $request->user_transfer ? 1 : 0;
        $general->payment_withdrawal = $request->payment_withdrawal ? 1 : 0;
        $general->crypto = $request->crypto ? 1 : 0;

        $general->livitpay = $request->livitpay ? 1 : 0;

        $general->save();

        $notify[] = ['success', 'Information has been updated.'];
        return back()->withNotify($notify);
    }

    public function pointsIndex() {
        $pageTitle = 'Points Setting';
        $general = GeneralSetting::first();

        return view('admin.setting.points_setting', compact('pageTitle', 'general'));
    }

    public function apiUpdate(Request $request) {
        $request->validate([
            'public_key' => 'required|max:255',
            'private_key' => 'required|max:255',
            'merchant_id' => 'required|max:255'
        ]);

        $general = GeneralSetting::first();

        $general->fiat_api_key = $request->fiat_api_key;
        $general->crypto_api_key = $request->crypto_api_key;
        $general->public_key = $request->public_key;
        $general->private_key = $request->private_key;
        $general->merchant_id = $request->merchant_id;
        $general->reloadly_id = $request->reloadly_id;
        $general->reloadly_secret = $request->reloadly_secret;
        $general->stripefee = $request->stripefee;
        $general->stripePublishableKey = $request->stripePublishableKey;
        $general->stripeSecretKey = $request->stripeSecretKey;
        $general->onramper = $request->onramper;
        $general->reloadly = $request->reloadly;
        $general->flutterwave_secret = $request->flutterwave_secret;
        $general->flutterwave_public = $request->flutterwave_public;
        $general->flutterwave_merchant = $request->flutterwave_merchant;
        $general->ezpin_secret = $request->ezpin_secret;
        $general->ezpin_id = $request->ezpin_id;
        $general->tremendous_key = $request->tremendous_key;
        $general->tremendous_us = $request->tremendous_us;
        $general->tremendous_row = $request->tremendous_row;
        $general->tremendous_mode = $request->tremendous_mode;
        $general->livitpay_public = $request->livitpay_public;
        $general->livitpay_private = $request->livitpay_private;
        $general->verifai = isset($request->verifai) ? $request->verifai : null;
        $general->vonageapikey = $request->vonageapikey;
        $general->vonagesecret = $request->vonagesecret;

        $general->save();

        $notify[] = ['success', 'Information has been updated.'];
        return back()->withNotify($notify);
    }

    public function moduleUpdate(Request $request) {


        $general = GeneralSetting::first();
        $general->force_ssl = $request->force_ssl ? 1 : 0;
        $general->secure_password = $request->secure_password ? 1 : 0;
        $general->registration = $request->registration ? 1 : 0;
        $general->agree = $request->agree ? 1 : 0;
        $general->tremendous = $request->tremendous ? 1 : 0;

        $general->force_ssl = $request->force_ssl ? 1 : 0;
        $general->ev = $request->ev ? 1 : 0;
        $general->en = $request->en ? 1 : 0;
        $general->sv = $request->sv ? 1 : 0;
        $general->sn = $request->sn ? 1 : 0;
        $general->cd = $request->cd ? 1 : 0;
        $general->loan = $request->loan ? 1 : 0;
        $general->dps = $request->dps ? 1 : 0;
        $general->fdr = $request->fdr ? 1 : 0;
        $general->vonage = $request->vonage ? 1 : 0;
        $general->investments = $request->investments ? 1 : 0;
        $general->games = $request->games ? 1 : 0;
        $general->remittance = $request->remittance ? 1 : 0;
        $general->crypto = $request->crypto ? 1 : 0;
        $general->va = $request->va ? 1 : 0;
        $general->physical = $request->physical ? 1 : 0;
        $general->livitpay = $request->livitpay ? 1 : 0;
        $general->verifai_act = $request->verifai_act ? 1 : 0;
        $general->kyc = $request->kyc ? 1 : 0;

        $general->save();

        $notify[] = ['success', 'Information has been updated.'];
        return back()->withNotify($notify);
    }

    public function pointsUpdate(Request $request) {
        $request->validate([
            'fiat_minpoints' => 'required|integer|gt:0',
            'fiat_perpoints' => 'required|integer|gt:0',
            'crypto_perpoints' => 'required|integer|gt:0',
            'redeem_perpoints' => 'required|integer|gt:0',
            'redeem_minpoints' => 'required|integer|gt:0',
            'referred_by' => 'required|integer|gt:0',
            'referred_to' => 'required|integer|gt:0',
            'signup_bonus' => 'required|integer|gt:0',
        ]);

        $general = GeneralSetting::first();
        $general->points = $request->points ? 1 : 0;
        $general->fiat_minpoints = isset($request->fiat_minpoints) ? $request->fiat_minpoints : null;
        $general->fiat_perpoints = isset($request->fiat_perpoints) ? $request->fiat_perpoints : null;
        $general->crypto_perpoints = isset($request->crypto_perpoints) ? $request->crypto_perpoints : null;
        $general->redeem_minpoints = isset($request->redeem_minpoints) ? $request->redeem_minpoints : null;
        $general->redeem_perpoints = isset($request->redeem_perpoints) ? $request->redeem_perpoints : null;
        $general->referred_by = isset($request->referred_by) ? $request->referred_by : null;
        $general->referred_to = isset($request->referred_to) ? $request->referred_to : null;
        $general->signup_bonus = isset($request->signup_bonus) ? $request->signup_bonus : null;

        $general->save();

        $notify[] = ['success', 'Information has been updated.'];
        return back()->withNotify($notify);
    }

    //Holiday
    public function holiday() {
        $holidays = Holiday::paginate(getPaginate());
        $pageTitle = 'Holidays';
        return view('admin.setting.holidays', compact('holidays', 'pageTitle'));
    }

    public function holidaySubmit(Request $request) {
        $request->validate([
            'date' => 'required|date'
        ]);
        $holiday = new Holiday();
        $holiday->date = $request->date;
        $holiday->save();
        $notify[] = ['success', 'Holiday added succssfully'];
        return back()->withNotify($notify);
    }

    public function remove($id) {
        $holiday = Holiday::findOrFail($id);
        $holiday->delete();
        $notify[] = ['success', 'Holiday deleted succssfully'];
        return back()->withNotify($notify);
    }

    public function offDaySubmit(Request $request) {
        $general = GeneralSetting::first();
        $general->off_day = $request->off_day;
        $general->save();
        $notify[] = ['success', 'Weekly Holiday Setting Updated'];
        return back()->withNotify($notify);
    }

}
