<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Models\AdminNotification;
use App\Models\GeneralSetting;
use App\Models\User;
use App\Models\Points;
use App\Models\UserLogin;
use Illuminate\Auth\Events\Registered;
use Illuminate\Foundation\Auth\RegistersUsers;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rules\Password;
use App\Models\Accounting;
use App\Lib\Verifai;
use Carbon\Carbon;

class RegisterController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Register Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles the registration of new users as well as their
    | validation and creation. By default this controller uses a trait to
    | provide this functionality without requiring any additional code.
    |
    */

    use RegistersUsers;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest');
        $this->middleware('regStatus')->except('registrationNotAllowed');

        $this->activeTemplate = activeTemplate();
    }

    public function showRegistrationForm()
    {
        $pageTitle = "Sign Up";
        $info = json_decode(json_encode(getIpInfo()), true);
        $mobile_code = @implode(',', $info['code']);
        $countries = json_decode(file_get_contents(resource_path('views/partials/country.json')));
        return view($this->activeTemplate . 'user.auth.register', compact('pageTitle','mobile_code','countries'));
    }


    /**
     * Get a validator for an incoming registration request.
     *
     * @param  array $data
     * @return \Illuminate\Contracts\Validation\Validator
     */
    protected function validator(array $data)
    {
        $general = GeneralSetting::first();
        $password_validation = Password::min(6);
        if ($general->secure_password) {
            $password_validation = $password_validation->mixedCase()->numbers()->symbols()->uncompromised();
        }
        $agree = 'nullable';
        if ($general->agree) {
            $agree = 'required';
        }
        $countryData = (array)json_decode(file_get_contents(resource_path('views/partials/country.json')));
        $countryCodes = implode(',', array_keys($countryData));
        $mobileCodes = implode(',',array_column($countryData, 'dial_code'));
        $countries = implode(',',array_column($countryData, 'country'));
        $validate = Validator::make($data, [
            'firstname' => 'sometimes|required|string|max:50',
            'lastname' => 'sometimes|required|string|max:50',
            'email' => 'required|string|email|max:90|unique:users',
            'profile'=> 'required|string',
            'mobile' => 'required|string|max:50|unique:users',
            'password' => ['required','confirmed',$password_validation],
            'username' => 'required|alpha_num|unique:users|min:6',
            'captcha' => 'sometimes|required',
            'mobile_code' => 'required|in:'.$mobileCodes,
            'country_code' => 'required|in:'.$countryCodes,
            'country' => 'required|in:'.$countries,
            'agree' => $agree
        ]);
        return $validate;
    }

    public function register(Request $request)
    {
        $this->validator($request->all())->validate();
        $exist = User::where('mobile',$request->mobile_code.$request->mobile)->first();
        if ($exist) {
            $notify[] = ['error', 'The mobile number already exists'];
            return back()->withNotify($notify)->withInput();
        }

        if (isset($request->captcha)) {
            if (!captchaVerify($request->captcha, $request->captcha_secret)) {
                $notify[] = ['error', "Invalid captcha"];
                return back()->withNotify($notify)->withInput();
            }
        }

        event(new Registered($user = $this->create($request->all())));

        $this->guard()->login($user);

        return $this->registered($request, $user)
            ?: redirect($this->redirectPath());
    }


    /**
     * Create a new user instance after a valid registration.
     *
     * @param  array $data
     * @return \App\User
     */
    protected function create(array $data)
    {

        $general = GeneralSetting::first();
        

        $referBy = session()->get('reference');
        if ($referBy) {
            $referUser = User::where('username', $referBy)->first();
        } else {
            $referUser = null;
        }
        if(isset($referUser->id)){
             $points = new Points();
                            $referUser->point_balance += $general->referred_by;
                            $referUser->save();
                            $points->user_id = $referUser->id;
                           
                            $points->amount = $general->referred_by;
                            $points->post_balance = $referUser->point_balance;
                            $points->charge = "0";
                            $points->trx_type = '+';
                            $points->details = 'Referral Points Earned Username '.trim($data['username']);
                            $points->trx = getTrx();
                            $points->save();
        }
        //User Create
        $user = new User();
        $user->firstname = isset($data['firstname']) ? $data['firstname'] : null;
        $user->lastname = isset($data['lastname']) ? $data['lastname'] : null;
        $user->email = strtolower(trim($data['email']));
        $user->company = isset($data['company']) ? $data['company'] : null;
        $user->password = Hash::make($data['password']);
        $user->username = trim($data['username']);
        $user->profile = strtolower($data['profile']);
        $user->ref_by = $referUser ? $referUser->id : 0;
        $user->country_code = $data['country_code'];
        $user->mobile = $data['mobile_code'].$data['mobile'];
        $user->address = [
            'address' => '',
            'state' => '',
            'zip' => '',
            'country' => isset($data['country']) ? $data['country'] : null,
            'city' => ''
        ];
        $user->status = 1;
        $user->ev = $general->ev ? 0 : 1;
        $user->sv = $general->sv ? 0 : 1;
        $user->ts = 0;
        $user->tv = 1;
        $user->ewallet_id = $general->sitename.rand(111111,999999);
        $user->save();

        if(isset($referUser->id)){
        $points = new Points();
                            $user->point_balance += $general->referred_to;
                            $user->save();
                            $points->user_id = $user->id;
                           
                            $points->amount = $general->referred_to;
                            $points->post_balance = $user->point_balance;
                            $points->charge = "0";
                            $points->trx_type = '+';
                            $points->details = 'Referral Points Earned From '.$referUser->username;
                            $points->trx = getTrx();
                            $points->save();
        
        } 
        if($general->signup_bonus){
            $points = new Points();
                            $user->point_balance += $general->signup_bonus;
                            $user->save();
                            $points->user_id = $user->id;
                           
                            $points->amount = $general->signup_bonus;
                            $points->post_balance = $user->point_balance;
                            $points->charge = "0";
                            $points->trx_type = '+';
                            $points->details = 'Sign Up Bonus Points';
                            $points->trx = getTrx();
                            $points->save();
        }
        
        $adminNotification = new AdminNotification();
        $adminNotification->user_id = $user->id;
        $adminNotification->title = 'New member registered';
        $adminNotification->click_url = urlPath('admin.users.detail',$user->id);
        $adminNotification->save();
        $hash = md5(rand(0,1000));
        
         $accounts = new Accounting;
         $accounts->setConnection('mysql_acc'); // non-static method
         $accounts->name = isset($data['firstname']) ? $data['firstname'] : null;
         $accounts->user_name = trim($data['username']);
         $accounts->slug = trim($data['username']);
         $accounts->email = strtolower(trim($data['email']));
         $accounts->verify_code = $hash;
         $accounts->thumb = null;
         $accounts->password = Hash::make($data['password']);
         $accounts->role = "user";
         $accounts->account_type = "pro";
         $accounts->user_type = "registered";
         $accounts->trial_expire = date_format(Carbon::now(),"Y-m-d");;
         $accounts->status = 1;
         $accounts->created_at = date_format(Carbon::now(),"Y-m-d H:i:s");
         $accounts->save();
             
        

        //Login Log Create
        $ip = $_SERVER["REMOTE_ADDR"];
        $exist = UserLogin::where('user_ip',$ip)->first();
        $userLogin = new UserLogin();

        //Check exist or not
        if ($exist) {
            $userLogin->longitude =  $exist->longitude;
            $userLogin->latitude =  $exist->latitude;
            $userLogin->city =  $exist->city;
            $userLogin->country_code = $exist->country_code;
            $userLogin->country =  $exist->country;
        }else{
            $info = json_decode(json_encode(getIpInfo()), true);
            $userLogin->longitude =  @implode(',',$info['long']);
            $userLogin->latitude =  @implode(',',$info['lat']);
            $userLogin->city =  @implode(',',$info['city']);
            $userLogin->country_code = @implode(',',$info['code']);
            $userLogin->country =  @implode(',', $info['country']);
        }

        $userAgent = osBrowser();
        $userLogin->user_id = $user->id;
        $userLogin->user_ip =  $ip;
        
        $userLogin->browser = @$userAgent['browser'];
        $userLogin->os = @$userAgent['os_platform'];
        $userLogin->save();


        return $user;
    }

    public function checkUser(Request $request){
        $exist['data'] = null;
        $exist['type'] = null;
        if ($request->email) {
            $exist['data'] = User::where('email',$request->email)->first();
            $exist['type'] = 'email';
        }
        if ($request->mobile) {
            $exist['data'] = User::where('mobile',$request->mobile)->first();
            $exist['type'] = 'mobile';
        }
        if ($request->username) {
            $exist['data'] = User::where('username',$request->username)->first();
            $exist['type'] = 'username';
        }
        return response($exist);
    }

    public function registered()
    {
        
          $user = auth()->user();
          $general = GeneralSetting::first();
           if($general->verifai_act != 0){
          if($user->profile == "merchant" && empty($user->verifai)){
          $veri = new Verifai();
          $response = $veri->create();
$user->verifai = $response->uuid;
$user->verifai_status = $response->status->state;
$user->save();
return redirect()->away($response->url);
          
          } else {
               return redirect()->route('user.home');
          }
        
    } else {
        return redirect()->route('user.home');
    }

}
}