<?php

namespace App\Http\Controllers;

use Carbon\Carbon;
use App\Models\Cart;
use App\Models\Fiat;
use App\Models\Page;
use App\Models\Plan;
use App\Models\Crypto;
use App\Models\Category;
use App\Models\Frontend;
use App\Models\Language;
use App\Models\FiatGateway;
use App\Models\SubCategory;
use App\Models\Transaction;
use App\Http\Helpers\Helper;
use Illuminate\Http\Request;
use App\Models\Advertisement;
use App\Models\SupportTicket;
use App\Models\GeneralSetting;
use App\Models\SupportMessage;
use App\Models\AdminNotification;
use App\Models\SupportAttachment;
use App\Models\GiftCardTransactions;
use Illuminate\Http\Resources\Json\JsonResource;
use Illuminate\Support\Facades\Auth;

class GiftCardController extends Controller
{
    public function __construct()
    {
        $this->activeTemplate = activeTemplate();
    }


    public function getToken()
    {
        sleep(3);
        $curl = curl_init();
        $general = GeneralSetting::first();
        $client_id = $general->reloadly_id;
        $client_secret = $general->reloadly_secret;
        $mode = $general->reloadly;
        $audience = ($mode == "live") ? "https://giftcards.reloadly.com" : "https://giftcards-sandbox.reloadly.com";

        curl_setopt_array($curl, [
            CURLOPT_URL => "https://auth.reloadly.com/oauth/token",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => '{
            "client_id":"' . $client_id . '",
            "client_secret":"' . $client_secret . '",
            "grant_type":"client_credentials",
            "audience":"' . $audience . '"}',
            CURLOPT_HTTPHEADER => [
                'Content-Type: application/json'
            ],
        ]);

        $response = curl_exec($curl);

        if ($response === false) {
            $error = curl_error($curl);
            // Handle the cURL error, log it, or return an appropriate response
            return $error;
        }

        $resp = json_decode($response);

        if (json_last_error() !== JSON_ERROR_NONE) {
            // Handle JSON decoding error, log it, or return an appropriate response
            return 'JSON decoding error: ' . json_last_error_msg();
        }

        if (!isset($resp->access_token)) {
            // Handle the case where the access_token property is not present
            return 'Access token not found in the response';
        }

        curl_close($curl);

        return $resp->access_token;
    }

    public function getCountries()
    {
        $token = $this->getToken();


        $curl = curl_init();
        $general = GeneralSetting::first();
        $client_id = $general->reloadly_id;
        $client_secret = $general->reloadly_secret;
        $mode = $general->reloadly;
        $audience = ($mode == "live") ? "https://giftcards.reloadly.com" : "https://giftcards-sandbox.reloadly.com";

        curl_setopt_array($curl, [
            CURLOPT_URL => $audience . '/countries',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_HTTPHEADER => [
                "Accept: application/com.reloadly.giftcards-v1+json",
                "Authorization: Bearer $token"
            ],
        ]);

        $response = curl_exec($curl);
        $getCountries = json_decode($response, true); // Decode the JSON response into an associative array
        curl_close($curl);

        return $getCountries;
    }


    function getExchangeRate($amount, $currenyCode)
    {
        $token = $this->getToken();
        $general = GeneralSetting::first();
        $client_id = $general->reloadly_id;
        $client_secret = $general->reloadly_secret;
        $mode = $general->reloadly;
        $audience = ($mode == "live") ? "https://giftcards.reloadly.com" : "https://giftcards-sandbox.reloadly.com";

        $curl = curl_init();
        $url = $audience . '/fx-rate?currencyCode=' . urlencode($currenyCode) . '&amount=' . urlencode($amount);

        curl_setopt_array($curl, [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_HTTPHEADER => [
                "Accept: application/com.reloadly.giftcards-v1+json",
                "Authorization: Bearer $token"
            ],
        ]);

        $response = curl_exec($curl);
        $getcurrency = json_decode($response, true);
        curl_close($curl);
        return $getcurrency;
    }



    public function getGiftCardProduct($token, $page)
    {
        $curl = curl_init();
        $general = GeneralSetting::first();
        $client_id = $general->reloadly_id;
        $client_secret = $general->reloadly_secret;
        $mode = $general->reloadly;
        $audience = ($mode == "live") ? "https://giftcards.reloadly.com" : "https://giftcards-sandbox.reloadly.com";
        $url = $audience . '/products?page=' . urlencode($page);
        curl_setopt_array($curl, [
            CURLOPT_URL => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_HTTPHEADER => [
                "Accept: application/com.reloadly.giftcards-v1+json",
                "Authorization: Bearer $token"
            ],
        ]);

        $response = curl_exec($curl);
        $giftCardProductsData = json_decode($response, true); // Decode the JSON response into an associative array
        curl_close($curl);

        // Print the response for debugging
        // dd($giftCardProductsData); die;

        // Calculate total items
        $totalItems = $giftCardProductsData['totalElements'];
        // dd($totalItems); die;

        // Calculate pagination offsets
        $perPage = $giftCardProductsData['numberOfElements'];
        $offset = ($page - 1) * $perPage;
        // dd( $offset); die;


        // Slice the data to get items for the current page
        $paginatedData = array($giftCardProductsData['content'], $offset, $perPage);
        // dd($paginatedData); die;

        return [
            'paginatedData' => $paginatedData,
            'totalItems' => $totalItems,
            'totalPages' => $giftCardProductsData['totalPages'],
            'currentPage' => $page,
        ];
    }

    public function getGiftCardProductById($productId)
    {

        $token = $this->getToken();
        $curl = curl_init();
        $general = GeneralSetting::first();
        $client_id = $general->reloadly_id;
        $client_secret = $general->reloadly_secret;
        $mode = $general->reloadly;
        $audience = ($mode == "live") ? "https://giftcards.reloadly.com" : "https://giftcards-sandbox.reloadly.com";
        curl_setopt_array($curl, [
            CURLOPT_URL => $audience . '/products/' . $productId,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_HTTPHEADER => [
                "Accept: application/com.reloadly.giftcards-v1+json",
                "Authorization: Bearer $token"
            ],
        ]);

        $response = curl_exec($curl);
        $giftCardProductsById = json_decode($response, true); // Decode the JSON response into an associative array
        curl_close($curl);

        return  $giftCardProductsById;
    }


  public function showGiftCardProducts(Request $request)
    {
        $pageTitle = 'Gift Cards';
        $emptyMessage = 'No data found';

        $regionName = $request->input('region_name');

        // Get access token
        $token = $this->getToken();
        $page = $request->input('page', 1);
        // Retrieve paginated gift card products data
        $giftCardProductsData = $this->getGiftCardProduct($token, $page);
if (empty($giftCardProductsData) || isset($giftCardProductsData['message']) || array_key_exists("errorCode",  $giftCardProductsData)) {
            


            $notify[] = ['error',  $giftCardProductsData["message"] ?? 'Something went wrong while processing Request'];
            return redirect()->back()->withNotify($notify);
        }
        $allGiftCards = $giftCardProductsData['paginatedData'][0];

        // Extract paginated data and other necessary information
        $totalItems = $giftCardProductsData['totalItems'];
        $totalPages = $giftCardProductsData['totalPages'];
        $currentPage = $giftCardProductsData['currentPage'];

        // If a region is specified, retrieve all gift card products without pagination
        if ($regionName) {
            // Initialize an array to store all gift card products
            $allGiftCards = [];

            // Iterate over each page to collect all gift card products
            for ($page = 1; $page <= $totalPages; $page++) {
                // Retrieve gift card products data for the current page
                $giftCardProductsData = $this->getGiftCardProduct($token, $page);

                // Extract gift card products from the response
                $giftCardProducts = $giftCardProductsData['paginatedData'][0];

                // Filter gift card products based on the specified region
                $filteredGiftCards = collect($giftCardProducts)->filter(function ($product) use ($regionName) {
                    return $product['country']['isoName'] === $regionName;
                })->all(); // Convert the filtered collection back to array

                // Merge the filtered gift card products into the array of all gift cards
                $allGiftCards = array_merge($allGiftCards, $filteredGiftCards);
            }
        }

        // Convert the merged array to a collection
        $giftCardProducts = collect($allGiftCards);

        // Get countries for filtering
        $countries = $this->getCountries();

        return view($this->activeTemplate . 'user.reloadly_giftcard.reloadly_gift_card', compact('pageTitle', 'emptyMessage', 'giftCardProducts', 'countries', 'totalItems', 'totalPages', 'currentPage'));
    }
    public function details($productId)
    {
        // Assuming $id is the product ID you want to fetch details for
        $giftCardProductsById = $this->getGiftCardProductById($productId);
        if (empty($giftCardProductsById) || isset($giftCardProductsById['message']) || array_key_exists("errorCode",  $giftCardProductsById)) {
            $result_set = array(
                "status" => 'error',
                "code" => 400,
                "message" =>    $giftCardProductsById["message"] ?? 'Something went wrong while processing Request.',
                "data" => array(
                    "error_code" =>   $giftCardProductsById["errorCode"],
                    "error_message" =>    $giftCardProductsById["message"] ?? null,
                    "error" =>    $giftCardProductsById["errorCode"]
                )
            );


            $notify[] = ['error',  $giftCardProductsById["message"] ?? 'Something went wrong while processing Request'];
            return redirect()->back()->withNotify($notify);
        } else {
            $pageTitle = 'Gift Card Details';
            $emptyMessage = 'No data found';
            $giftCardProducts = $giftCardProductsById;

            // Fetch reloadly fees from the general settings
            $general = GeneralSetting::first();

            // Pass the 'fixedRecipientToSenderDenominationsMap' to the view
            $fixedRecipientToSenderDenominationsMap = $giftCardProducts['fixedRecipientToSenderDenominationsMap'];

            return view(
                $this->activeTemplate . 'user.reloadly_giftcard.reloadly_gift_card_details',
                compact('giftCardProducts', 'pageTitle', 'fixedRecipientToSenderDenominationsMap', 'general', 'emptyMessage')
            );
        }
    }



    public function processGiftCardOrder(Request $request)
    {
        $request_data = $request->input();

        if (
            empty($request_data['product_id']) ||
            empty($request_data['amount']) ||
            empty($request_data['currency_code']) ||
            empty($request_data['quantity']) ||
            !isset($request_data['logo_urls']) ||
            empty($request_data['logo_urls']) ||
            !is_numeric($request_data['amount']) ||
            !is_numeric($request_data['quantity']) ||
            intval($request_data['amount']) <= 0 ||
            intval($request_data['quantity']) <= 0
        ) {

            $notify[] = ['error', 'Bad Request: Invalid or missing parameters'];
            return back()->withNotify($notify);
        }
        $currenyCode = $request_data['currency_code'];

        $logoUrls = $request_data['logo_urls'];

        // Convert the array of logo URLs into a single string
        $logoUrlsString = implode(',', $logoUrls);


        $amount = (float)$request_data['amount'];


        $general = GeneralSetting::first();
        $user = Auth::user();
        // Call the getExchangeRate method with the amount and currency code
        $exchangeRate = $this->getExchangeRate($amount, $currenyCode);
        // dd($exchangeRate); die;
        $senderAmount = $exchangeRate['senderAmount'];
        $senderCurrency = $exchangeRate['senderCurrency'];
        $recipientCurrency = $exchangeRate['recipientCurrency'];
        $total_price = $exchangeRate['senderAmount'] * $request_data['quantity'];

        $totalCost = ($exchangeRate['senderAmount'] * $request_data['quantity']) + ($general->reloadly_fee * $request_data['quantity']);
        $general->reloadly_fee;
        //    dd($totalCost); die;

        if ($totalCost > $user->balance) {
            $notify[] = ['error', 'Sorry, Insufficient Balance to process this order. Required Balance USD' . $totalCost];
            return back()->withNotify($notify);
        }




        // Generate transaction ID
        $trx = getTrx();

        // Deduct total cost from user's balance
        $user->balance -= $totalCost;
        $user->save();

        // Prepare request parameters for gift card order
        $request_parameters = array(
            "productId" => (int)$request_data['product_id'],
            "quantity" => (int)$request_data['quantity'],
            "unitPrice" => $amount,
            "customIdentifier" => $trx,
            "senderName" => $user->username,
            "recipientEmail" => $user->email,
            "recipientPhoneDetails" => array(
                "countryCode" => $user->country_code,
                "phoneNumber" => $user->mobile
            )
        );

        // Get access token
        $getToken = $this->getToken();
        $getTokenAccess = $getToken;
        // Place gift card order
        $giftcard_api_response = $this->giftCardOrder($getTokenAccess, $request_parameters);
        // dd($giftcard_api_response); die;


        // Check if API response is empty
        if (empty($giftcard_api_response) || isset($giftcard_api_response['message']) || array_key_exists("errorCode", $giftcard_api_response)) {

            $result_set = array(
                "status" => 'error',
                "code" => 400,
                "message" => $giftcard_api_response["message"] ?? 'Something went wrong while processing Request.',
                "data" => array(
                    "error_code" => $giftcard_api_response["errorCode"],
                    "error_message" => $giftcard_api_response["message"] ?? null,
                    "error" => $giftcard_api_response["errorCode"]
                )
            );
            $user->balance += $totalCost;
            $user->save();
            $itransaction = new Transaction();
            $itransaction->user_id = $user->id;
            $itransaction->amount = $totalCost; // You can set the amount to the total cost
            $itransaction->charge = 0;
            $itransaction->trx_type = '+';
            $itransaction->trx = getTrx();
            $itransaction->details = 'Reversal For GiftCard Purchase ';
            $itransaction->post_balance = $user->balance;
            $itransaction->save();


            $notify[] = ['error', 'Error Completing Purchase at this time, Try later'];
            return back()->withNotify($notify);
        }

        $result_set = array(
            "status" => 'success',
            "code" => 200,
            "message" => 'Gift card purchased successfully!',
            "data" => array(
                "response" => $giftcard_api_response,
            )
        );




        // Extract relevant data from the response
        if ($result_set['status'] === 'success') {

            $itransaction = new Transaction();
            $itransaction->user_id = $user->id;
            $itransaction->amount = $totalCost;
            $itransaction->charge = 0;
            $itransaction->trx_type = '-';
            $itransaction->trx = getTrx();
            $itransaction->details = 'Gift card purchased successfully! ';
            $itransaction->post_balance = $user->balance;
            $itransaction->save();

            $transactionData = [
                'user_id' => $user->id,
                'transaction_id' => isset($giftcard_api_response['transactionId']) ? $giftcard_api_response['transactionId'] : 0,
                'amount' =>  $totalCost,
                'discount' => $giftcard_api_response['discount'],
                'currency_code' => $senderCurrency,
                'fee' => $giftcard_api_response['fee'],
                'total_fee' => $giftcard_api_response['totalFee'],
                'recipient_email' => $giftcard_api_response['recipientEmail'],
                'custom_identifier' => $giftcard_api_response['customIdentifier'],
                'status' => $giftcard_api_response['status'],
                'logo_urls' => $logoUrlsString,
                'product_id' => $giftcard_api_response['product']['productId'],
                'product_name' => $giftcard_api_response['product']['productName'],
                'country_code' => $giftcard_api_response['product']['countryCode'],
                'product_currency_code' => $recipientCurrency,
                'quantity' => $giftcard_api_response['product']['quantity'],
                'unit_price' => $senderAmount,
                'total_price' => $total_price,
                'brand_id' => $giftcard_api_response['product']['brand']['brandId'],
                'brand_name' => $giftcard_api_response['product']['brand']['brandName'],
                'sms_fee' => $giftcard_api_response['smsFee'],
                'recipient_phone' => $giftcard_api_response['recipientPhone'],
                'transaction_created_time' => $giftcard_api_response['transactionCreatedTime'],
                'pre_ordered' => $giftcard_api_response['preOrdered'],
            ];

            GiftCardTransactions::create($transactionData);
            // Redirect back with the appropriate notification
            $notify[] = ['success', 'Gift card purchased successfully!'];
            return redirect()->back()->withNotify($notify);
        } else {

            $user->balance += $totalCost;
            $user->save();
            $itransaction = new Transaction();
            $itransaction->user_id = $user->id;
            $itransaction->amount = $totalCost;
            $itransaction->charge = 0;
            $itransaction->trx_type = '+';
            $itransaction->trx = getTrx();
            $itransaction->details = 'Reversal For GiftCard Purchase ';
            $itransaction->post_balance = $user->balance;
            $itransaction->save();


            $notify[] = ['error', 'Failed to purchase gift card. Please try again.'];
            return redirect()->back()->withNotify($notify);
        }
    }

    public function getGiftCardPurchasedHistory()
    {
        $pageTitle = 'Purchased Gift Card ';
        $emptyMessage = 'Data Not Found';

        $user = Auth::user();
        //Fetch purchase history of the user's gift cards
        $purchaseHistory = GiftCardTransactions::where('user_id', $user->id)->get();

        // Check if purchase history is empty
        if ($purchaseHistory->isEmpty()) {
            $notify[] = ['error', 'No purchase History available for you.'];
            return redirect()->back()->withNotify($notify);
        }

        return view($this->activeTemplate . 'user.reloadly_giftcard.purchased_gift_card',  compact('pageTitle', 'emptyMessage', 'purchaseHistory'));
    }

    public function getPurchasedCardDetails($product_id)
    {
        $pageTitle = 'Purchased Gift Card Details';
        $emptyMessage = 'Data Not Found';

        // Retrieve the authenticated user
        $user = Auth::user();

        // Fetch purchase details of the user's gift cards for the specified product_id
        $purchaseCardDetails = GiftCardTransactions::where('user_id', $user->id)
            ->where('product_id', $product_id)
            ->first();

        // Check if the purchase details exist
        if (!$purchaseCardDetails) {
            $notify[] = ['error', 'No purchase details found for the specified product.'];
            return redirect()->back()->withNotify($notify);
        }

        // Extract brand ID and transaction ID from purchase details
        $brandId = $purchaseCardDetails->brand_id;
        $transactionId = $purchaseCardDetails->transaction_id;

        // Call the first API to get redeem instructions by brand ID
        $redeemInstructions = $this->getRedeemInstructionByBrandId($brandId);
        if (empty($redeemInstructions) || isset($redeemInstructions['message']) || array_key_exists("errorCode", $redeemInstructions)) {

            $notify[] = ['error', $redeemInstructions["message"] ?? 'Failed to fetch redeem instructions'];
            return redirect()->back()->withNotify($notify);
        }

        // Call the second API to get redeem codes by transaction ID
        $redeemCodes = $this->getGiftCardRedeemCode($transactionId);
        if (empty($redeemCodes) || isset($redeemCodes['message']) || array_key_exists("errorCode", $redeemCodes)) {
            $notify[] = ['error', $redeemCodes["message"] ?? 'Failed to fetch redeem codes'];
            return redirect()->back()->withNotify($notify);
        }


        // Prepare the result data
        $result = [
            'brand_id' => $brandId,
            'transaction_id' => $transactionId,
            'redeem_instructions' => $redeemInstructions,
            'redeem_codes' => $redeemCodes
        ];
        // var_dump($result); die;

        // Return the view with the result data
        return view($this->activeTemplate . 'user.reloadly_giftcard.purchased_gift_card_details', compact('pageTitle', 'emptyMessage', 'result'));
    }

    public function getRedeemInstructionByBrandId($brandId)
    {

        $token = $this->getToken();
        $curl = curl_init();
        $general = GeneralSetting::first();
        $mode = $general->reloadly;
        $audience = ($mode == "live") ? "https://giftcards.reloadly.com" : "https://giftcards-sandbox.reloadly.com";
        curl_setopt_array($curl, [
            CURLOPT_URL => $audience . '/brands/' . $brandId . '/redeem-instructions',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_HTTPHEADER => [
                "Accept: application/com.reloadly.giftcards-v1+json",
                "Authorization: Bearer $token"
            ],
        ]);

        $response = curl_exec($curl);
        $err = curl_error($curl);

        curl_close($curl);

        if ($err) {
            return "cURL Error #:" . $err;
        } else {
            //Decode the JSON string
            $decodedResponse = json_decode($response, true);

            return $decodedResponse;
        }
    }


    public function getGiftCardRedeemCode($transactionId)
    {

        $token = $this->getToken();
        $curl = curl_init();
        $general = GeneralSetting::first();
        $mode = $general->reloadly;
        $audience = ($mode == "live") ? "https://giftcards.reloadly.com" : "https://giftcards-sandbox.reloadly.com";
        curl_setopt_array($curl, [
            CURLOPT_URL => $audience . '/orders/transactions/' . $transactionId . '/cards',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "GET",
            CURLOPT_HTTPHEADER => [
                "Accept: application/com.reloadly.giftcards-v1+json",
                "Authorization: Bearer $token"
            ],
        ]);

        $response = curl_exec($curl);
        $err = curl_error($curl);

        curl_close($curl);

        if ($err) {
            return "cURL Error #:" . $err;
        } else {
            //Decode the JSON string
            $decodedResponse = json_decode($response, true);
            return $decodedResponse;
        }
    }
    public function giftCardOrder($token, $request_parameters)
    {
        $curl = curl_init();
        $general = GeneralSetting::first();

        // Convert request parameters array to JSON
        $data = json_encode($request_parameters);

        $mode = $general->reloadly;
        $audience = ($mode == "live") ? "https://giftcards.reloadly.com" : "https://giftcards-sandbox.reloadly.com";
        curl_setopt_array($curl, [
            CURLOPT_URL => $audience . '/orders',
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => $data, // Set JSON-encoded request body
            CURLOPT_HTTPHEADER => [
                "Accept: application/com.reloadly.giftcards-v1+json",
                "Authorization: Bearer $token",
                "Content-Type: application/json"
            ],
        ]);

        $response = curl_exec($curl);
        $err = curl_error($curl);

        if ($err) {
            // Handle cURL error
            // You may want to log the error instead of echoing it
            echo "cURL Error #:" . $err;
            return null; // Return null to indicate an error
        }

        // Decode the JSON response
        $decoded_response = json_decode($response, true);

        // Check if decoding was successful
        if ($decoded_response === null) {
            // Handle JSON decoding error
            echo "Error decoding JSON response";
            return null; // Return null to indicate an error
        }

        // Check if the API response indicates an error
        if (isset($decoded_response['status']) && $decoded_response['status'] === 'error') {
            // Handle API error
            echo "API Error: " . $decoded_response['message'];
            return null; // Return null to indicate an error
        }

        // Return the decoded response if everything is successful
        return $decoded_response;
    }
}
