<?php

namespace App\Http\Controllers;

use App\Http\Controllers\Controller;
use App\Lib\ProcessSendMoney;
use App\Models\Country;
use App\Models\GatewayCurrency;
use App\Models\SendingPurpose;
use App\Models\SendMoney;
use App\Models\SourceOfFund;
use Illuminate\Http\Request;

class UserSendMoneyController extends Controller
{
    /*
     * Return Send Money View Page
     */
    public function __construct()
    {
        $this->activeTemplate = activeTemplate();
    }
    public function sendMoney()
    {
        $pageTitle        = 'Global Money Transfer';
        $countries        = Country::active()->get();
        $sources          = SourceOfFund::active()->get();
        $purposes         = SendingPurpose::active()->get();

        $sessionData      = session()->get('send_money') ?? [];
        $recipientCountry = null;

        if ($sessionData) {
            $sendingCountry   = $countries->where('id', $sessionData['sending_country'])->first()->id;
            $recipientCountry = $countries->where('id', $sessionData['recipient_country'])->first()->id;
        } else {
            $ipInfo           = json_decode(json_encode(getIpInfo()), true);
            $countryName      = @implode(',', $ipInfo['country']);
            $sendingCountry   = @$countries->where('name', $countryName)->first()->id ?? @$countries->first()->id;
        }

        if (!$recipientCountry) {
            $recipientCountry = @$countries->where('id', '!=', $sendingCountry)->first()->id;
        }

        $sendingAmount      = @$sessionData['sending_amount'];
        $recipientAmount    = @$sessionData['recipient_amount'];

        session()->forget('send_money');
        return view($this->activeTemplate . 'user.send_money.form', compact('pageTitle', 'countries', 'sources', 'purposes', 'sendingAmount', 'recipientAmount', 'sendingCountry', 'recipientCountry'));
    }

    /*
     * Initially save the send-money data
     */
    public function save(Request $request)
    {
        $request->validate(
            [
                'sending_amount'    => 'required|numeric|gt:0',
                'sending_country'   => 'required|gt:0',
                'recipient_country' => 'required|gt:0',
                'payment_type'      => 'required|in:1,2',
                'source_of_funds'   => 'required|gt:0',
                'sending_purpose'   => 'required|gt:0',
                'recipient'         => 'required|array|min:3',
                'recipient.*'       => 'required|string',
            ],
            [
                'recipient.name.required'    => 'Recipient name field is required',
                'recipient.mobile.required'  => 'Recipient mobile number field is required',
                'recipient.address.required' => 'Recipient address field is required',
                'recipient.bank.required' => 'Recipient bank field is required',
                'recipient.account.required' => 'Recipient IBAN/AC/ACH field is required',
                'recipient.routing.required' => 'Recipient Routing/IFSC/BIC/SwiftCode field is required',
                'recipient.additional.required' => 'Mention any Additional Info like Bank Address,etc',
            ]
        );


        $user                = auth()->user();
        $payment             = new ProcessSendMoney();
        $payment->user       = $user;
        $payment->columnName = 'user_id';

        $sendMoney           = $payment->createSendMoney($request);


        if ($request->payment_type == 1) {

            if ($payment->amountWithCharge > $user->balance) {
                $notify[] = ['error', 'Insufficient Balance.'];
            } else {
                $notify[] = ['success', 'Send money request submitted successfully'];
                $payment->createTransaction();
                ProcessSendMoney::updateSendMoney($sendMoney, $user);
            }
            return redirect()->route('user.send.money.history')->withNotify($notify);
        }

        session()->put('payment_trx', $sendMoney->trx);
          return back()->withNotify($notify);
    }

    /*
     * Redirect to payment page
     */
    public function payNow()
    {
        $gatewayCurrency = GatewayCurrency::whereHas('method', function ($gate) {
            $gate->where('status', 1);
        })->with('method')->orderby('method_code')->get();

        $pageTitle = 'Pay Money';
        $trx       = session()->get('payment_trx');
        $sendMoney = SendMoney::filterUser()->where('trx', $trx)->first();
        if (!$sendMoney) {
            $notify[] = ['error', 'Session invalidate'];
              return back()->withNotify($notify);
        }

        return view($this->activeTemplate . 'user.payment.deposit', compact('gatewayCurrency', 'pageTitle', 'sendMoney'));
    }

    /*
     * Redirect to payment page to pay for previously initialized send-money
     */
    public function pay(Request $request)
    {
        $sendMoney = SendMoney::filterUser()->initiated()->findOrFail(decrypt($request->id));
        session()->put('payment_trx', $sendMoney->trx);
        return redirect()->route('user.send.money.pay.now');
    }

    /*
     * Transfer History
     */
    public function history()
    {
        $pageTitle    = 'MoneyTransfer Transactions';
        $emptyMessage = 'No transfers found';
        $transfers    = SendMoney::filterUser()->latest()->paginate(getPaginate());

        return view($this->activeTemplate . 'user.send_money.history', compact('pageTitle', 'emptyMessage', 'transfers'));
    }
}
